#!/usr/local/bin/perl -w
# File:     sample_postfilter.pl
# Created:  3-Apr-1998

# COPYRIGHT 1998,1999
# bCandid Corporation.
# All Rights Reserved.
#
# software@bcandid.com
# 303-545-5550

# This sample filter for posts accepts most article.  It will, however,
# reject any article with "MAKE MONEY FAST" as its Subject, and
# will edit the articles posted by authenticated users to add an
# X-Sender header.

# Flush STDOUT
$| = 1;
$previous_line = "\r\n";

# Enter a simple state machine
while (defined($line = <>)) {

    # If we just completed the article send a response
    if ($previous_line =~ /\r\n$/ && $line eq ".\r\n") {
	# If the Subject field has "MAKE MONEY FAST", forget it!
	if ($subject =~ m/MAKE MONEY FAST/) {
	    print "435\r\n.\r\n";
	    $subject = $username = $hostname = $ipaddress =
		$in_header = $in_body = 0;
	    @hlines = @blines = ();
	    $previous_line = $line;
	    next;
	}
	
	# Otherwise, we are accepting it
	print "235\r\n";

	# If the user authenticated, provide a NEW version of the article
	#  with the X-Sender field
	if ($username) {
	    $xsender = "X-Sender: $username@";
	    if ($hostname) {
		$xsender .= "$hostname\r\n";
	    } else {
		$xsender .= "$ipaddress\r\n";
	    }
	    # Output new header, and destroy any existing X-Sender header
	    $found_xsender = 0;
	    foreach $h (@hlines) {
		if ($h =~ m/^X-Sender:/) {
		    print $xsender;
		    $found_xsender = 1;
		} else {
		    print $h;
		}
	    }
	    if (!$found_xsender) {
		print $xsender;
	    }
	    print "\r\n";
	    foreach $b (@blines) {
		print $b;
	    }
	}

	# Terminate the response
	print ".\r\n";
	$subject = $username = $hostname = $ipaddress = $in_header = 
	    $in_body = 0;
	@hlines = @blines = ();
    }

    # We are fed:
    #   FeedInformationLines\r\n
    #   \r\n
    #   HeaderLines\r\n
    #   \r\n
    #   BodyLines\r\n
    #   .\r\n
    #
    # So, set up simple state to save the right lines
    #

    # If we are in the header and we see the Subject field, save it.
    if ($in_header) {
	if ($line =~ m/^Subject: (.*)\r\n$/) {
	    $subject = $1;
	}
    }

    # If we are in the FeedInformation area, grab Authentication
    #  information for later use
    if (!$in_header && !$in_body) {
	if ($line =~ m/^Username: (.*)\r\n$/) {
	    $username = $1;
	}
	if ($line =~ m/^Hostname: (.*)\r\n$/) {
	    $hostname = $1;
	}
	if ($line =~ m/^IPAddress: (.*)\r\n$/) {
	    $ipaddress = $1;
	}
	if ($line =~ m/^Cookie: (.*)\r\n$/) {
	    $cookie = $1;
	    if ($cookie) {};
	}
	if ($line =~ m/^ConnectionTag: (.*)\r\n$/) {
	    $connectiontag = $1;
	    if ($connectiontag) {};
	}
	if ($line =~ m/^SessionID: (.*)\r\n$/) {
	    $sessionid = $1;
	    if ($sessionid) {};
	}
    }

    if ($line eq "\r\n") {
	if ($in_body) {
	    push @blines, $line;
	}
	if ($in_header) {
	    $in_body = 1;
	} else {
	    $in_header = 1;
	}
    } elsif ($in_body) {
	push @blines, $line;
    } elsif ($in_header) {
	push @hlines, $line;
    }

    # Save last line for use in end of transaction detection
    $previous_line = $line;
}
